
# q-expansion of newform 1600.4.a.ck, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1600
weight = 4
poly_data = [-19, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [0, 1], [0, 0], [0, -1], [20, 0], [0, 6], [0, -8], [84, 0], [0, -7], [6, 0], [224, 0], [0, 14], [266, 0], [0, -35], [0, 43], [0, 42], [28, 0], [-182, 0], [0, -49], [-408, 0], [0, 124], [48, 0], [0, -23], [1526, 0], [0, -64], [-1246, 0], [0, 97], [0, 147], [902, 0], [0, 168], [0, -301], [2940, 0], [0, 84], [364, 0], [254, 0], [2360, 0], [0, -254], [0, -175], [0, 351], [0, -218], [1972, 0], [1330, 0], [-1728, 0], [0, 112], [0, -434], [-1512, 0], [1644, 0], [0, 125], [0, -113], [3934, 0], [0, -532], [3856, 0], [994, 0], [-6300, 0], [0, -768], [0, 161], [-5082, 0], [-2800, 0], [0, 182], [-1254, 0], [0, 533], [0, -750], [0, -729], [5208, 0], [0, -524], [0, 98], [7828, 0], [0, 616], [0, 315], [-8890, 0], [0, 176], [9144, 0], [0, -1189], [0, 378], [2588, 0], [0, -467], [-5426, 0], [0, -1126], [370, 0], [-4186, 0], [-2716, 0], [-1966, 0], [11824, 0], [0, -296], [-13272, 0], [0, 357], [2094, 0], [0, -812], [9450, 0], [0, 357], [0, 615], [13216, 0], [0, -273], [-12300, 0], [-15756, 0], [0, -2199], [-2394, 0], [6874, 0], [0, 1853], [-7174, 0], [0, -497], [0, 1050], [0, -1591], [16070, 0], [-3932, 0], [0, -1856], [0, -1293], [0, -1224], [-6840, 0], [-10150, 0], [0, 211], [0, -574], [0, -2828], [27020, 0], [10648, 0], [1682, 0], [0, -2127], [0, 3519], [0, -154], [-21356, 0], [13762, 0], [0, -672], [0, -3394], [0, 1981], [-1652, 0], [-28790, 0], [-7554, 0], [28336, 0], [0, -2697], [0, -386], [-14300, 0], [0, 3633], [11824, 0], [0, -154], [6762, 0], [-29442, 0], [0, -4110], [0, 2743], [0, -1966], [8646, 0], [-18956, 0], [-13726, 0], [0, -5271], [0, -245], [-8778, 0], [-18088, 0], [0, -2630], [0, -516], [-40516, 0], [0, 3213], [0, -3878], [19362, 0], [0, -567], [0, -5049], [0, -4893], [15888, 0], [7944, 0], [20622, 0], [0, 4724], [-40502, 0], [0, -441], [0, 3668], [0, 2191], [17556, 0], [0, 504], [0, -1079], [-28576, 0], [0, -3618], [43310, 0], [0, -726], [-14436, 0], [53466, 0], [59368, 0], [0, -196], [48944, 0], [-50442, 0], [-5692, 0], [55266, 0], [0, 6785], [-59066, 0], [0, 3731], [57148, 0], [0, 2674], [0, -4380], [0, -1211], [-210, 0], [0, 98], [0, -4927], [-13642, 0], [11648, 0], [0, 6816], [0, -707], [-8660, 0], [49770, 0], [0, 8127], [0, 4908], [56034, 0], [0, 4158], [0, 2400], [0, 4465], [11286, 0], [-76272, 0], [0, 5766], [34846, 0], [13580, 0], [0, -6222], [17472, 0], [0, -4655], [-13274, 0], [-25004, 0], [0, -5992], [-5502, 0], [0, -4823], [0, -6693], [45752, 0], [17290, 0], [0, 1435], [-49742, 0], [0, 1351], [0, -10066], [-20526, 0], [77672, 0], [-87618, 0], [0, -4907], [0, -8457], [-20802, 0], [0, -2540], [-76000, 0], [0, -1889], [85556, 0], [0, -4298], [57652, 0], [73792, 0], [43434, 0], [0, -6131], [0, -11605], [-83202, 0], [0, 10906], [17340, 0], [54600, 0], [0, 11193], [-59164, 0], [0, 7265], [-99370, 0], [0, -2120], [62664, 0], [0, -765], [116508, 0], [-58036, 0], [0, -11851], [0, 1890], [-9982, 0], [0, -2177], [114646, 0], [0, -5738], [89748, 0], [-43822, 0], [0, 11867], [0, 6574], [0, -820], [0, -9891], [0, -12929], [18494, 0], [0, 15070], [0, -6144], [-18844, 0], [85686, 0], [109802, 0], [0, 2149], [0, 10626], [30058, 0], [0, -10969], [0, 5972], [82784, 0], [0, 184], [0, -2759], [0, 7259], [38950, 0], [-67222, 0], [25452, 0], [0, -5811], [-113336, 0], [0, 3799], [3458, 0], [0, -5293], [-61424, 0], [0, 4872], [0, -6650], [-8120, 0], [96782, 0], [-80198, 0], [0, -4089], [0, -924], [21588, 0], [0, 18998], [-26922, 0], [-12768, 0], [0, 8826], [82460, 0], [0, -2913], [0, 7820], [0, 10122], [-47888, 0], [0, -8631], [49412, 0], [0, 6496], [0, 11123], [18550, 0], [-84376, 0], [0, 13986], [0, -6123], [106350, 0], [-63390, 0], [0, 2961], [0, 13055], [56966, 0], [72436, 0], [99968, 0], [0, -608], [-74178, 0], [-532, 0], [0, 20636], [-138390, 0], [0, -1267], [0, 19740], [69986, 0], [-125628, 0], [0, 2805], [0, -7693], [0, 98], [139130, 0], [0, 3794], [-29120, 0], [0, -10639], [-112236, 0], [0, 15147], [-135850, 0], [0, -10320], [22666, 0], [0, -10261], [0, -20454], [0, 12236], [-181874, 0], [-131480, 0], [0, -1666], [-18972, 0], [-53550, 0], [0, -16093], [173936, 0], [0, 246], [25340, 0], [0, 6132], [192858, 0], [0, -7659], [1966, 0], [0, 18324], [141792, 0], [-136332, 0], [0, 17416], [0, 18137], [0, 21518], [19866, 0], [0, 22827], [-156484, 0], [0, 9375], [0, -18452], [0, 10410], [0, 25473], [235290, 0], [-41252, 0], [-170548, 0], [0, 8309], [50910, 0], [14616, 0], [0, 18690], [57428, 0], [130208, 0], [0, 15408], [-134946, 0], [0, -25684], [57498, 0], [0, -15316], [0, -4697], [0, -4816], [142884, 0], [0, -19167], [15408, 0], [0, 14470], [0, 1701], [0, 15779], [-265810, 0], [0, -20574], [108396, 0], [0, 19175], [272504, 0], [0, 4228], [-103712, 0], [193606, 0], [-32908, 0], [81954, 0], [-144970, 0], [0, -7280], [0, -4949], [0, 6684], [-235410, 0], [106120, 0], [0, -27510], [82578, 0], [0, -9319], [-271740, 0], [0, -8392], [0, 26166], [0, -24171], [178012, 0], [0, 34132], [190666, 0], [176896, 0], [0, 5199], [0, 15640], [0, -1527], [150870, 0], [0, 2014], [0, -9877], [174300, 0], [0, -16372], [0, -31254], [0, 3689], [87526, 0], [-200508, 0], [-147224, 0]]
